# coding: utf-8

# Copyright 2019, 2024 IBM All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64

from requests import Request

from ibm_cloud_sdk_core.logger import get_logger
from .authenticator import Authenticator
from ..utils import has_bad_first_or_last_char

logger = get_logger()


class BasicAuthenticator(Authenticator):
    """The BasicAuthenticator is used to add basic authentication information to requests.

    Basic Authorization will be sent as an Authorization header in the form:

        Authorization: Basic <encoded username and password>

    Args:
        username: User-supplied username for basic auth.
        password: User-supplied password for basic auth.

    Raises:
        ValueError: The username or password is not specified or contains invalid characters.
    """

    def __init__(self, username: str, password: str) -> None:
        self.username = username
        self.password = password
        self.validate()
        self.authorization_header = self.__construct_basic_auth_header()
        logger.debug('Created new BasicAuthenticator instance!')

    def authentication_type(self) -> str:
        """Returns this authenticator's type ('basic')."""
        return Authenticator.AUTHTYPE_BASIC

    def validate(self) -> None:
        """Validate username and password.

        Ensure the username and password are valid for service operations.

        Raises:
            ValueError: The username and/or password is not valid for service operations.
        """
        if self.username is None or self.password is None:
            raise ValueError('The username and password shouldn\'t be None.')

        if has_bad_first_or_last_char(self.username) or has_bad_first_or_last_char(self.password):
            raise ValueError(
                'The username and password shouldn\'t start or end with curly brackets or quotes. '
                'Please remove any surrounding {, }, or \" characters.'
            )

    def __construct_basic_auth_header(self) -> str:
        authstring = "{0}:{1}".format(self.username, self.password)
        base64_authorization = base64.b64encode(authstring.encode('utf-8')).decode('utf-8')
        return 'Basic {0}'.format(base64_authorization)

    def authenticate(self, req: Request) -> None:
        """Add basic authentication information to a request.

        Basic Authorization will be added to the request's headers in the form:

            Authorization: Basic <encoded username and password>

        Args:
            req: The request to add basic auth information to. Must contain a key to a dictionary
            called headers.
        """
        headers = req.get('headers')
        headers['Authorization'] = self.authorization_header
        logger.debug('Authenticated outbound request (type=%s)', self.authentication_type())



# coding: utf-8

# Copyright 2019, 2025 IBM All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from .authenticators import (
    Authenticator,
    BasicAuthenticator,
    BearerTokenAuthenticator,
    ContainerAuthenticator,
    CloudPakForDataAuthenticator,
    IAMAuthenticator,
    IAMAssumeAuthenticator,
    NoAuthAuthenticator,
    VPCInstanceAuthenticator,
    MCSPAuthenticator,
    MCSPV2Authenticator,
)
from .utils import read_external_sources, string_to_bool
from .logger import get_logger

logger = get_logger()


def get_authenticator_from_environment(service_name: str) -> Authenticator:
    """Look for external configuration of authenticator.

    Try to get authenticator from external sources, with the following priority:
    1. Credentials file(ibm-credentials.env)
    2. Environment variables
    3. VCAP Services(Cloud Foundry)

    Args:
        service_name: The service name.

    Returns:
        The authenticator found from service information.
    """
    logger.debug('Get authenticator from environment, key=%s', service_name)
    authenticator = None
    config = read_external_sources(service_name)
    if config:
        authenticator = __construct_authenticator(config)
    if authenticator is not None:
        logger.debug('Returning authenticator, type=%s', authenticator.authentication_type())
    return authenticator


# pylint: disable=too-many-branches
# pylint: disable=too-many-statements
def __construct_authenticator(config: dict) -> Authenticator:
    # Determine the authentication type if not specified explicitly.
    if config.get('AUTH_TYPE'):
        auth_type = config.get('AUTH_TYPE')
    elif config.get('AUTHTYPE'):
        auth_type = config.get('AUTHTYPE')
    else:
        # If authtype wasn't specified explicitly, then determine the default.
        # If the APIKEY property is specified, then it should be IAM, otherwise Container Auth.
        if config.get('APIKEY'):
            auth_type = Authenticator.AUTHTYPE_IAM
        else:
            auth_type = Authenticator.AUTHTYPE_CONTAINER

    auth_type = auth_type.lower()
    authenticator = None

    if auth_type == Authenticator.AUTHTYPE_BASIC.lower():
        authenticator = BasicAuthenticator(username=config.get('USERNAME'), password=config.get('PASSWORD'))
    elif auth_type == Authenticator.AUTHTYPE_BEARERTOKEN.lower():
        authenticator = BearerTokenAuthenticator(bearer_token=config.get('BEARER_TOKEN'))
    elif auth_type == Authenticator.AUTHTYPE_CONTAINER.lower():
        authenticator = ContainerAuthenticator(
            cr_token_filename=config.get('CR_TOKEN_FILENAME'),
            iam_profile_name=config.get('IAM_PROFILE_NAME'),
            iam_profile_id=config.get('IAM_PROFILE_ID'),
            url=config.get('AUTH_URL'),
            client_id=config.get('CLIENT_ID'),
            client_secret=config.get('CLIENT_SECRET'),
            disable_ssl_verification=string_to_bool(config.get('AUTH_DISABLE_SSL', 'false')),
            scope=config.get('SCOPE'),
        )
    elif auth_type == Authenticator.AUTHTYPE_CP4D.lower():
        authenticator = CloudPakForDataAuthenticator(
            username=config.get('USERNAME'),
            password=config.get('PASSWORD'),
            url=config.get('AUTH_URL'),
            apikey=config.get('APIKEY'),
            disable_ssl_verification=string_to_bool(config.get('AUTH_DISABLE_SSL', 'false')),
        )
    elif auth_type == Authenticator.AUTHTYPE_IAM.lower() and config.get('APIKEY'):
        authenticator = IAMAuthenticator(
            apikey=config.get('APIKEY'),
            url=config.get('AUTH_URL'),
            client_id=config.get('CLIENT_ID'),
            client_secret=config.get('CLIENT_SECRET'),
            disable_ssl_verification=string_to_bool(config.get('AUTH_DISABLE_SSL', 'false')),
            scope=config.get('SCOPE'),
        )
    elif auth_type == Authenticator.AUTHTYPE_IAM_ASSUME.lower():
        authenticator = IAMAssumeAuthenticator(
            apikey=config.get('APIKEY'),
            iam_profile_id=config.get('IAM_PROFILE_ID'),
            iam_profile_crn=config.get('IAM_PROFILE_CRN'),
            iam_profile_name=config.get('IAM_PROFILE_NAME'),
            iam_account_id=config.get('IAM_ACCOUNT_ID'),
            url=config.get('AUTH_URL'),
            client_id=config.get('CLIENT_ID'),
            client_secret=config.get('CLIENT_SECRET'),
            disable_ssl_verification=string_to_bool(config.get('AUTH_DISABLE_SSL', 'false')),
            scope=config.get('SCOPE'),
        )
    elif auth_type == Authenticator.AUTHTYPE_VPC.lower():
        authenticator = VPCInstanceAuthenticator(
            iam_profile_crn=config.get('IAM_PROFILE_CRN'),
            iam_profile_id=config.get('IAM_PROFILE_ID'),
            url=config.get('AUTH_URL'),
        )
    elif auth_type == Authenticator.AUTHTYPE_MCSP.lower():
        authenticator = MCSPAuthenticator(
            apikey=config.get('APIKEY'),
            url=config.get('AUTH_URL'),
        )
    elif auth_type == Authenticator.AUTHTYPE_MCSPV2.lower():
        # Required arguments.
        apikey = config.get('APIKEY')
        url = config.get('AUTH_URL')
        scope_collection_type = config.get('SCOPE_COLLECTION_TYPE')
        scope_id = config.get('SCOPE_ID')

        # Optional arguments.
        optional_args = {}
        str_value = config.get("INCLUDE_BUILTIN_ACTIONS")
        if str_value is not None:
            optional_args['include_builtin_actions'] = string_to_bool(str_value)

        str_value = config.get("INCLUDE_CUSTOM_ACTIONS")
        if str_value is not None:
            optional_args['include_custom_actions'] = string_to_bool(str_value)

        str_value = config.get("INCLUDE_ROLES")
        if str_value is not None:
            optional_args['include_roles'] = string_to_bool(str_value)

        str_value = config.get("PREFIX_ROLES")
        if str_value is not None:
            optional_args['prefix_roles'] = string_to_bool(str_value)

        str_value = config.get("CALLER_EXT_CLAIM")
        if str_value is not None:
            try:
                optional_args['caller_ext_claim'] = json.loads(str_value)
            except Exception as caused_by:
                msg = 'An error occurred while unmarshalling the CALLER_EXT_CLAIM configuration property: {0}'.format(
                    str_value
                )
                raise ValueError(msg) from caused_by

        str_value = config.get("AUTH_DISABLE_SSL")
        if str_value is not None:
            optional_args['disable_ssl_verification'] = string_to_bool(str_value)

        authenticator = MCSPV2Authenticator(
            apikey=apikey, url=url, scope_collection_type=scope_collection_type, scope_id=scope_id, **optional_args
        )
    elif auth_type == Authenticator.AUTHTYPE_NOAUTH.lower():
        authenticator = NoAuthAuthenticator()

    return authenticator



# coding: utf-8
# Copyright 2019 IBM All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Classes and helper functions used by generated SDKs.

classes:
    BaseService: Abstract class for common functionality between each service.
    DetailedResponse: The object returned from successful service operations.
    IAMTokenManager: Requests and refreshes IAM tokens using an apikey, and optionally a client_id and client_secret.
    IAMAssumeTokenManager: Requests and refreshes IAM tokens using an apikey and a trusted profile.
    JWTTokenManager: Abstract class for common functionality between each JWT token manager.
    CP4DTokenManager: Requests and refreshes CP4D tokens given a username and password.
    ApiException: Custom exception class for errors returned from service operations.

functions:
    datetime_to_string: Serializes a datetime to a string.
    string_to_datetime: De-serializes a string to a datetime.
    datetime_to_string_list: Serializes a list of datetimes to a list of strings.
    string_to_datetime_list: De-serializes a list of strings to a list of datetimes.
    date_to_string: Serializes a date to a string.
    string_to_date: De-serializes a string to a date.
    convert_model: Convert a model object into an equivalent dict.
    convert_list: Convert a list of strings into comma-separated string.
    get_query_param: Return a query parameter value from a URL
    read_external_sources: Get config object from external sources.
    get_authenticator_from_environment: Get authenticator from external sources.
"""

from .base_service import BaseService
from .detailed_response import DetailedResponse
from .token_managers.iam_token_manager import IAMTokenManager
from .token_managers.iam_assume_token_manager import IAMAssumeTokenManager
from .token_managers.jwt_token_manager import JWTTokenManager
from .token_managers.cp4d_token_manager import CP4DTokenManager
from .token_managers.container_token_manager import ContainerTokenManager
from .token_managers.vpc_instance_token_manager import VPCInstanceTokenManager
from .token_managers.mcsp_token_manager import MCSPTokenManager
from .token_managers.mcspv2_token_manager import MCSPV2TokenManager
from .api_exception import ApiException
from .utils import datetime_to_string, string_to_datetime, read_external_sources
from .utils import datetime_to_string_list, string_to_datetime_list
from .utils import date_to_string, string_to_date
from .utils import convert_model, convert_list
from .utils import get_query_param
from .get_authenticator import get_authenticator_from_environment



# coding: utf-8

# Copyright 2019, 2024 IBM All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from requests import Request

from ibm_cloud_sdk_core.logger import get_logger
from .authenticator import Authenticator

logger = get_logger()


class BearerTokenAuthenticator(Authenticator):
    """The BearerTokenAuthenticator will add a user-supplied bearer token
    to requests.

    The bearer token will be sent as an Authorization header in the form:

         Authorization: Bearer <bearer-token>

    Args:
        bearer_token: The user supplied bearer token.

    Raises:
        ValueError: Bearer token is none.
    """

    def __init__(self, bearer_token: str) -> None:
        self.bearer_token = bearer_token
        self.validate()
        logger.debug('Created BearerTokenAuthenticator instance!')

    def authentication_type(self) -> str:
        """Returns this authenticator's type ('bearertoken')."""
        return Authenticator.AUTHTYPE_BEARERTOKEN

    def validate(self) -> None:
        """Validate the bearer token.

        Ensures the bearer token is valid for service operations.

        Raises:
            ValueError: The bearer token is not valid for service operations.
        """
        if self.bearer_token is None:
            raise ValueError('The bearer token shouldn\'t be None.')

    def authenticate(self, req: Request) -> None:
        """Adds bearer authentication information to the request.

        The bearer token will be added to the request's headers in the form:

            Authorization: Bearer <bearer-token>

        Args:
            req: The request to add bearer authentication information to. Must contain a key to a dictionary
            called headers.
        """
        headers = req.get('headers')
        headers['Authorization'] = 'Bearer {0}'.format(self.bearer_token)
        logger.debug('Authenticated outbound request (type=%s)', self.authentication_type())

    def set_bearer_token(self, bearer_token: str) -> None:
        """Set a new bearer token to be sent in subsequent service operations.

        Args:
            bearer_token: The bearer token that will be sent in service requests.

        Raises:
            ValueError: The bearer token is not valid for service operations.
        """
        self.bearer_token = bearer_token
        self.validate()
